/*
 * Copyright (C) 2022-2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.lock;

import java.time.Instant;
import java.time.temporal.ChronoUnit;

import de.ozgcloud.common.logging.OzgCloudLogging;
import de.ozgcloud.user.common.errorhandling.LockCreationException;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import lombok.extern.log4j.Log4j2;

@Log4j2
@ApplicationScoped
@OzgCloudLogging
public class LockService {

	@Inject
	LockRepository repository;

	public boolean isNotLocked() {
		return repository.count() == 0;
	}

	public boolean isLockOlderThanADay() {
		return repository.findLockBefore(Instant.now())
				.map(this::isSyncLockOlderThanADay)
				.orElse(false);
	}

	private boolean isSyncLockOlderThanADay(Lock syncLock) {
		var lastSyncTimeStamp = Instant.ofEpochMilli(syncLock.getTimestamp());
		return lastSyncTimeStamp.plus(1, ChronoUnit.DAYS).isBefore(Instant.now());
	}

	public void unlock() {
		var count = repository.deleteAll();
		LOG.info("%s Locks deleted".formatted(count));
	}

	public void lock(Instant timestamp) {
		repository.persist(Lock.builder().timestamp(timestamp.toEpochMilli()).build());
		LOG.info("Lock created");
	}

	Lock getByTimestamp(Instant timestamp) {
		return repository.findLock(timestamp).orElseThrow(LockCreationException::new);
	}

	public Lock lockCollection() {
		var now = Instant.now();
		lock(now);

		return getByTimestamp(now);
	}
}
